@@ -40,7 +40,7 @@ import           Data.Word (Word8, Word16)
 import           System.IO.Unsafe (unsafePerformIO)
 import           Data.Typeable (Typeable)
 
-import Data.Conduit hiding (Source, Conduit, Sink, Pipe)
+import Data.Conduit
 import qualified Data.Conduit.List as CL
 import Control.Monad.Trans.Class (lift)
 import Control.Monad (unless)
@@ -67,15 +67,15 @@ instance Show Codec where
 -- | Emit each line separately
 --
 -- Since 0.4.1
-lines :: Monad m => GInfConduit T.Text m T.Text
+lines :: Monad m => Conduit T.Text m T.Text
 lines =
     loop id
   where
-    loop front = awaitE >>= either (finish front) (go front)
+    loop front = await >>= maybe (finish front) (go front)
 
-    finish front r =
+    finish front =
         let final = front T.empty
-         in unless (T.null final) (yield final) >> return r
+         in unless (T.null final) (yield final)
 
     go sofar more =
         case T.uncons second of
@@ -90,7 +90,7 @@ lines =
 -- not capable of representing an input character, an exception will be thrown.
 --
 -- Since 0.3.0
-encode :: MonadThrow m => Codec -> GInfConduit T.Text m B.ByteString
+encode :: MonadThrow m => Codec -> Conduit T.Text m B.ByteString
 encode codec = CL.mapM $ \t -> do
     let (bs, mexc) = codecEncode codec t
     maybe (return bs) (monadThrow . fst) mexc
@@ -100,15 +100,15 @@ encode codec = CL.mapM $ \t -> do
 -- not capable of decoding an input byte sequence, an exception will be thrown.
 --
 -- Since 0.3.0
-decode :: MonadThrow m => Codec -> GInfConduit B.ByteString m T.Text
+decode :: MonadThrow m => Codec -> Conduit B.ByteString m T.Text
 decode codec =
     loop id
   where
-    loop front = awaitE >>= either (finish front) (go front)
+    loop front = await >>= maybe (finish front) (go front)
 
-    finish front r =
+    finish front =
         case B.uncons $ front B.empty of
-            Nothing -> return r
+            Nothing -> return ()
             Just (w, _) -> lift $ monadThrow $ DecodeException codec w
 
     go front bs' =
